@@ -4,15 +4,15 @@ Developed by P Angerer, see https://github.com/flying-sheep/get_version.
 """
 # __version__ is defined at the very end of this file.
 
-import re
 import os
+import re
 from pathlib import Path
-from subprocess import run, PIPE, CalledProcessError
-from typing import NamedTuple, List, Union, Optional
-
+from subprocess import PIPE, CalledProcessError, run
+from typing import List, NamedTuple, Optional, Union
 
 RE_VERSION = r"([\d.]+?)(?:\.dev(\d+))?(?:[_+-]([0-9a-zA-Z.]+))?"
-RE_GIT_DESCRIBE = r"v?(?:([\d.]+)-(\d+)-g)?([0-9a-f]{7})(-dirty)?"
+# RE_GIT_DESCRIBE = r"v?(?:([\d.]+)-(\d+)-g)?([0-9a-f]{7})(-dirty)?"
+RE_GIT_DESCRIBE = r"v?(?:([\d.]+)-(.+)-g)?([0-9a-f]{7})(-dirty)?"
 ON_RTD = os.environ.get("READTHEDOCS") == "True"
 
 
@@ -86,9 +86,7 @@ def get_version_from_git(parent):
         check=True,
     )
 
-    release, dev, hex_, dirty = match_groups(
-        f"{RE_GIT_DESCRIBE}$", p.stdout.rstrip("\r\n")
-    )
+    release, dev, hex_, dirty = match_groups(f"{RE_GIT_DESCRIBE}", p.stdout.rstrip("\r\n"))
 
     labels = []
     if dev == "0":
@@ -104,7 +102,7 @@ def get_version_from_git(parent):
 
 def get_version_from_metadata(name: str, parent: Optional[Path] = None):
     try:
-        from pkg_resources import get_distribution, DistributionNotFound
+        from pkg_resources import DistributionNotFound, get_distribution
     except ImportError:
         return None
 
@@ -158,28 +156,41 @@ def get_version(package: Union[Path, str]) -> str:
         parent = path.parent
 
     return str(
-        get_version_from_dirname(name, parent)
+        get_dynamo_version()
+        or get_version_from_dirname(name, parent)
         or get_version_from_git(parent)
         or get_version_from_metadata(name, parent)
-        or "0.0.0"
     )
 
 
+def get_dynamo_version():
+    import pkg_resources
+
+    try:
+        _package_name = "dynamo-release"
+        _package = pkg_resources.working_set.by_key[_package_name]
+        version = _package.version
+    except KeyError:
+        version = "1.0.9"
+
+    return version
+
+
 def get_all_dependencies_version(display=True):
     """
     Adapted from answer 2 in
     https://stackoverflow.com/questions/40428931/package-for-listing-version-of-packages-used-in-a-jupyter-notebook
     """
+    import pandas as pd
     import pkg_resources
     from IPython.display import display
-    import pandas as pd
 
-    _package_name = 'dynamo-release'
+    _package_name = "dynamo-release"
     _package = pkg_resources.working_set.by_key[_package_name]
 
-    all_dependencies = ([str(r).split('>')[0] for r in _package.requires()])  # retrieve deps from setup.py
+    all_dependencies = [str(r).split(">")[0] for r in _package.requires()]  # retrieve deps from setup.py
     all_dependencies.sort(reverse=True)
-    all_dependencies.insert(0, 'dynamo-release')
+    all_dependencies.insert(0, "dynamo-release")
 
     all_dependencies_list = []
 
@@ -187,17 +198,15 @@ def get_all_dependencies_version(display=True):
         if m.project_name.lower() in all_dependencies:
             all_dependencies_list.append([m.project_name, m.version])
 
-    df = pd.DataFrame(
-                    all_dependencies_list[::-1],
-                    columns=["package", "version"]
-                ).set_index("package").T
+    df = pd.DataFrame(all_dependencies_list[::-1], columns=["package", "version"]).set_index("package").T
 
     if display:
-        pd.options.display.max_columns=None
+        pd.options.display.max_columns = None
         display(df)
     else:
         return df
 
+
 __version__ = get_version(__file__)
 
 
