@@ -9,27 +9,34 @@ The code base will be extended extensively to consider the following cases:
     6. others
 """
 
-from ..tools.utils import update_dict
-from .utils import (
-    _select_font_color,
-    _get_extent,
-    _embed_datashader_in_an_axis,
-    _datashade_points,
-    save_fig,
-)
 
-from .utils import is_list_of_lists  # is_gene_name
+from typing import Optional, Union
+from warnings import warn
+
+import numpy as np
+import pandas as pd
+import scipy
+from anndata import AnnData
+from matplotlib.axes import Axes
+from matplotlib.figure import Figure
+
 from ..configuration import _themes
 from ..docrep import DocstringProcessor
-
-import pandas as pd
-import numpy as np
-from warnings import warn
+from ..tools.connectivity import check_and_recompute_neighbors
+from ..tools.utils import update_dict
+from .utils import is_list_of_lists  # is_gene_name
+from .utils import (
+    _datashade_points,
+    _embed_datashader_in_an_axis,
+    _get_extent,
+    _select_font_color,
+    save_fig,
+)
 
 docstrings = DocstringProcessor()
 
 
-def _plt_connectivity(coord, connectivity):
+def _plt_connectivity(coord: dict, connectivity: scipy.sparse.csr_matrix):
     """Plot connectivity graph via networkx and matplotlib.
 
     Parameters
@@ -44,8 +51,8 @@ def _plt_connectivity(coord, connectivity):
         Nothing but a connectivity graph plot built upon networkx and matplotlib.
     """
 
-    import networkx as nx
     import matplotlib.pyplot as plt
+    import networkx as nx
 
     if_symmetric = (abs(connectivity - connectivity.T) > 1e-10).nnz == 0
 
@@ -84,26 +91,26 @@ def _plt_connectivity(coord, connectivity):
 
 @docstrings.get_sectionsf("con_base")
 def connectivity_base(
-    x,
-    y,
-    edge_df,
-    highlights=None,
-    edge_bundling=None,
-    edge_cmap="gray_r",
-    show_points=True,
-    labels=None,
-    values=None,
-    theme=None,
-    cmap="Blues",
-    color_key=None,
-    color_key_cmap="Spectral",
-    background="black",
-    figsize=(7, 5),
-    ax=None,
-    sort='raw',
-    save_show_or_return='return',
-    save_kwargs={},
-):
+    x: int,
+    y: int,
+    edge_df: pd.DataFrame,
+    highlights: Optional[list] = None,
+    edge_bundling: Optional[str] = None,
+    edge_cmap: str = "gray_r",
+    show_points: bool = True,
+    labels: Optional[list] = None,
+    values: Optional[list] = None,
+    theme: Optional[str] = None,
+    cmap: str = "Blues",
+    color_key: Union[dict, list, None] = None,
+    color_key_cmap: str = "Spectral",
+    background: str = "black",
+    figsize: tuple = (7, 5),
+    ax: Optional[Axes] = None,
+    sort: str = "raw",
+    save_show_or_return: str = "return",
+    save_kwargs: dict = {},
+) -> Union[None, Axes]:
     """Plot connectivity relationships of the underlying UMAP
     simplicial set data structure. Internally UMAP will make
     use of what can be viewed as a weighted graph. This graph
@@ -218,16 +225,16 @@ def connectivity_base(
 
     Returns
     -------
-    result: matplotlib axis
-        The result is a matplotlib axis with the relevant plot displayed.
+    result:
+        Either return None or a matplotlib axis with the relevant plot displayed based on arguments.
         If you are using a notbooks and have ``%matplotlib inline`` set
         then this will simply display inline.
     """
 
-    import matplotlib.pyplot as plt
     import datashader as ds
-    import datashader.transfer_functions as tf
     import datashader.bundling as bd
+    import datashader.transfer_functions as tf
+    import matplotlib.pyplot as plt
 
     dpi = plt.rcParams["figure.dpi"]
 
@@ -262,10 +269,7 @@ def connectivity_base(
     if edge_bundling is None:
         edges = bd.directly_connect_edges(point_df, edge_df, weight="weight")
     elif edge_bundling == "hammer":
-        warn(
-            "Hammer edge bundling is expensive for large graphs!\n"
-            "This may take a long time to compute!"
-        )
+        warn("Hammer edge bundling is expensive for large graphs!\n" "This may take a long time to compute!")
         edges = bd.hammer_bundle(point_df, edge_df, weight="weight")
     else:
         raise ValueError("{} is not a recognised bundling method".format(edge_bundling))
@@ -308,8 +312,15 @@ def connectivity_base(
     ax.set(xticks=[], yticks=[])
 
     if save_show_or_return == "save":
-        s_kwargs = {"path": None, "prefix": 'connectivity_base', "dpi": None,
-                    "ext": 'pdf', "transparent": True, "close": True, "verbose": True}
+        s_kwargs = {
+            "path": None,
+            "prefix": "connectivity_base",
+            "dpi": None,
+            "ext": "pdf",
+            "transparent": True,
+            "close": True,
+            "verbose": True,
+        }
         s_kwargs = update_dict(s_kwargs, save_kwargs)
 
         save_fig(**s_kwargs)
@@ -325,29 +336,29 @@ docstrings.delete_params("con_base.parameters", "edge_df", "save_show_or_return"
 
 @docstrings.with_indent(4)
 def nneighbors(
-    adata,
-    x=0,
-    y=1,
-    color='ntr',
-    basis="umap",
-    layer="X",
-    highlights=None,
-    ncols=1,
-    edge_bundling=None,
-    edge_cmap="gray_r",
-    show_points=True,
-    labels=None,
-    values=None,
-    theme=None,
-    cmap=None,
-    color_key=None,
-    color_key_cmap=None,
-    background="black",
-    figsize=(6, 4),
-    ax=None,
-    save_show_or_return='show',
-    save_kwargs={},
-):
+    adata: AnnData,
+    x: int = 0,
+    y: int = 1,
+    color: str = "ntr",
+    basis: str = "umap",
+    layer: str = "X",
+    highlights: Union[list] = None,
+    ncols: int = 1,
+    edge_bundling: Optional[str] = None,
+    edge_cmap: str = "gray_r",
+    show_points: bool = True,
+    labels: Optional[list] = None,
+    values: Optional[list] = None,
+    theme: Optional[str] = None,
+    cmap: str = "Blues",
+    color_key: Optional[Union[dict, list]] = None,
+    color_key_cmap: str = "Spectral",
+    background: str = "black",
+    figsize: tuple = (6, 4),
+    ax: Optional[Axes] = None,
+    save_show_or_return: str = "return",
+    save_kwargs: dict = {},
+) -> Union[None, Figure]:
     """Plot nearest neighbor graph of cells used to embed data into low dimension space.
 
     Parameters
@@ -397,7 +408,10 @@ def nneighbors(
     # c_is_gene_name = [is_gene_name(adata, i) for i in list(color)] if n_c > 0 else [False] * n_c
     # cnt, gene_num = 0, sum(c_is_gene_name)
 
-    coo_graph = adata.uns["neighbors"]["connectivities"].tocoo()
+    check_and_recompute_neighbors(adata, result_prefix="")
+    # coo_graph = adata.uns["neighbors"]["connectivities"].tocoo()
+    coo_graph = adata.obsp["connectivities"].tocoo()
+
     edge_df = pd.DataFrame(
         np.vstack([coo_graph.row, coo_graph.col, coo_graph.data]).T,
         columns=("source", "target", "weight"),
@@ -476,13 +490,9 @@ def nneighbors(
                 # if highligts is a list of lists - each list is relate to each color element
                 if is_list_of_lists(highlights):
                     _highlights = highlights[color.index(cur_c)]
-                    _highlights = (
-                        _highlights if all([i in _color for i in _highlights]) else None
-                    )
+                    _highlights = _highlights if all([i in _color for i in _highlights]) else None
                 else:
-                    _highlights = (
-                        highlights if all([i in _color for i in highlights]) else None
-                    )
+                    _highlights = highlights if all([i in _color for i in highlights]) else None
 
                 connectivity_base(
                     x_,
@@ -503,13 +513,22 @@ def nneighbors(
                     ax,
                 )
 
-                ax.set_xlabel(cur_b + "_1",)
+                ax.set_xlabel(
+                    cur_b + "_1",
+                )
                 ax.set_ylabel(cur_b + "_2")
                 ax.set_title(cur_c)
 
     if save_show_or_return == "save":
-        s_kwargs = {"path": None, "prefix": 'nneighbors', "dpi": None,
-                    "ext": 'pdf', "transparent": True, "close": True, "verbose": True}
+        s_kwargs = {
+            "path": None,
+            "prefix": "nneighbors",
+            "dpi": None,
+            "ext": "pdf",
+            "transparent": True,
+            "close": True,
+            "verbose": True,
+        }
         s_kwargs = update_dict(s_kwargs, save_kwargs)
 
         save_fig(**s_kwargs)
