@@ -6,11 +6,12 @@ Created on Wed Jan 30 11:21:25 2019
 @author: xqiu
 """
 
-from .gillespie_utils import *
-from .bif_os_inclusive_sim import sim_diff, sim_osc, simulate, osc_diff_dup
+import anndata
 import pandas as pd
 import scipy.sparse
-from anndata import AnnData
+
+from .bif_os_inclusive_sim import osc_diff_dup, sim_diff, sim_osc, simulate
+from .utils import *
 
 
 # deterministic as well as noise
@@ -81,24 +82,21 @@ def Gillespie(
             steps = (t_span[1] - t_span[0]) // dt  # // int; %% remainder
             t_eval = np.linspace(t_span[0], t_span[1], steps)
         trajs_C = simulate_multigene(
-            a=None,
-            b=None,
-            la=None,
-            aa=None,
-            ai=None,
-            si=None,
-            be=None,
-            ga=None,
+            a=a,
+            b=b,
+            la=la,
+            aa=aa,
+            ai=ai,
+            si=si,
+            be=be,
+            ga=ga,
             C0=np.zeros((5, 1)),
             t_span=[0, 50],
             n_traj=1,
             t_eval=None,
             report=verbose,
         )  # unfinished, no need to interpolate now.
-        uu, ul, su, sl = [
-            np.transpose(trajs_C[:, :, i + 1, :].reshape((gene_num, -1)))
-            for i in range(4)
-        ]
+        uu, ul, su, sl = [np.transpose(trajs_C[:, :, i + 1, :].reshape((gene_num, -1))) for i in range(4)]
 
         u = uu + ul
         s = su + sl
@@ -261,9 +259,7 @@ def Gillespie(
 
         # synthesize steady state before treatment
         n_cell = 50
-        c0 = np.array(
-            [40, 100, 40, 100, 0, 0, 0, 0, 1000, 1000]
-        )  # same as the os model after this line of code
+        c0 = np.array([40, 100, 40, 100, 0, 0, 0, 0, 1000, 1000])  # same as the os model after this line of code
 
         n_species = len(c0)
         trajs_T, trajs_C = simulate(
@@ -274,9 +270,15 @@ def Gillespie(
             report=True,
         )
 
-        kin_5, kin_40, kin_200, kin_300, one_shot, deg_begin, deg_end = osc_diff_dup(
-            n_species, trajs_C, model_treat_lab, model_treat_unlab, n_cell
-        )
+        (
+            kin_5,
+            kin_40,
+            kin_200,
+            kin_300,
+            one_shot,
+            deg_begin,
+            deg_end,
+        ) = osc_diff_dup(n_species, trajs_C, model_treat_lab, model_treat_unlab, n_cell)
 
         uu = np.vstack(
             (
@@ -366,33 +368,25 @@ def Gillespie(
             [j for j in kin_Tl for i in range(n_cell)],
         )
         one_shot_cell_ids, one_shot_Trajectory, one_shot_Step = (
-            [
-                "one_shot_traj_%d_time_%d" % (i, j)
-                for j in kin_T_CP
-                for i in range(n_cell)
-            ],
+            ["one_shot_traj_%d_time_%d" % (i, j) for j in kin_T_CP for i in range(n_cell)],
             [i for j in kin_T_CP for i in range(n_cell)],
             [j for j in kin_T_CP for i in range(n_cell)],
         )  # first n_traj and then steps
         begin_cell_ids, begin_Trajectory, begin_Step = (
-            [
-                "begin_deg_traj_%d_time_%d" % (i, j)
-                for j in deg_label_t
-                for i in range(n_cell)
-            ],
+            ["begin_deg_traj_%d_time_%d" % (i, j) for j in deg_label_t for i in range(n_cell)],
             [i for j in deg_label_t for i in range(n_cell)],
             [j for j in deg_label_t for i in range(n_cell)],
         )  # first n_traj and then steps
         end_cell_ids, end_Trajectory, end_Step = (
-            [
-                "end_deg_traj_%d_time_%d" % (i, j)
-                for j in deg_label_t
-                for i in range(n_cell)
-            ],
+            ["end_deg_traj_%d_time_%d" % (i, j) for j in deg_label_t for i in range(n_cell)],
             [i for j in deg_label_t for i in range(n_cell)],
             [j for j in deg_label_t for i in range(n_cell)],
         )  # first n_traj and then steps
-        cell_ids, Trajectory, Step = kin_cell_ids * 4, kin_Trajectory * 4, kin_Step * 4
+        cell_ids, Trajectory, Step = (
+            kin_cell_ids * 4,
+            kin_Trajectory * 4,
+            kin_Step * 4,
+        )
         cell_ids.extend(one_shot_cell_ids)
         Trajectory.extend(one_shot_Trajectory)
         Step.extend(one_shot_Step)
@@ -514,12 +508,22 @@ def Gillespie(
 
         n_species = len(c0)
         trajs_T, trajs_C = simulate(
-            model_unlab, C0=[c0] * n_cell, t_span=[0, 100], n_traj=n_cell, report=True
+            model_unlab,
+            C0=[c0] * n_cell,
+            t_span=[0, 100],
+            n_traj=n_cell,
+            report=True,
         )
 
-        kin_5, kin_40, kin_200, kin_300, one_shot, deg_begin, deg_end = osc_diff_dup(
-            n_species, trajs_C, model_lab, model_unlab, n_cell
-        )
+        (
+            kin_5,
+            kin_40,
+            kin_200,
+            kin_300,
+            one_shot,
+            deg_begin,
+            deg_end,
+        ) = osc_diff_dup(n_species, trajs_C, model_lab, model_unlab, n_cell)
 
         uu = np.vstack(
             (
@@ -609,33 +613,25 @@ def Gillespie(
             [j for j in kin_Tl for i in range(n_cell)],
         )
         one_shot_cell_ids, one_shot_Trajectory, one_shot_Step = (
-            [
-                "one_shot_traj_%d_time_%d" % (i, j)
-                for j in kin_T_CP
-                for i in range(n_cell)
-            ],
+            ["one_shot_traj_%d_time_%d" % (i, j) for j in kin_T_CP for i in range(n_cell)],
             [i for j in kin_T_CP for i in range(n_cell)],
             [j for j in kin_T_CP for i in range(n_cell)],
         )  # first n_traj and then steps
         begin_cell_ids, begin_Trajectory, begin_Step = (
-            [
-                "begin_deg_traj_%d_time_%d" % (i, j)
-                for j in deg_label_t
-                for i in range(n_cell)
-            ],
+            ["begin_deg_traj_%d_time_%d" % (i, j) for j in deg_label_t for i in range(n_cell)],
             [i for j in deg_label_t for i in range(n_cell)],
             [j for j in deg_label_t for i in range(n_cell)],
         )  # first n_traj and then steps
         end_cell_ids, end_Trajectory, end_Step = (
-            [
-                "end_deg_traj_%d_time_%d" % (i, j)
-                for j in deg_label_t
-                for i in range(n_cell)
-            ],
+            ["end_deg_traj_%d_time_%d" % (i, j) for j in deg_label_t for i in range(n_cell)],
             [i for j in deg_label_t for i in range(n_cell)],
             [j for j in deg_label_t for i in range(n_cell)],
         )  # first n_traj and then steps
-        cell_ids, Trajectory, Step = kin_cell_ids * 4, kin_Trajectory * 4, kin_Step * 4
+        cell_ids, Trajectory, Step = (
+            kin_cell_ids * 4,
+            kin_Trajectory * 4,
+            kin_Step * 4,
+        )
         cell_ids.extend(one_shot_cell_ids)
         Trajectory.extend(one_shot_Trajectory)
         Step.extend(one_shot_Step)
@@ -695,13 +691,13 @@ def Gillespie(
     )  # use the real name in simulation?
     var.set_index("gene_short_name", inplace=True)
 
-    adata = AnnData(
+    adata = anndata.AnnData(
         scipy.sparse.csc_matrix(E.astype(int)).copy(),
         obs.copy(),
         var.copy(),
         layers=layers.copy(),
     )
-    adata_no_splicing = AnnData(
+    adata_no_splicing = anndata.AnnData(
         scipy.sparse.csc_matrix(E.astype(int)).copy(),
         obs.copy(),
         var.copy(),
@@ -712,8 +708,6 @@ def Gillespie(
         adata_no_splicing.obsm["protein"] = P
     # remove cells that has no expression
     adata = adata[np.array(adata.X.sum(1)).flatten() > 0, :]
-    adata_no_splicing = adata_no_splicing[
-        np.array(adata_no_splicing.X.sum(1)).flatten() > 0, :
-    ]
+    adata_no_splicing = adata_no_splicing[np.array(adata_no_splicing.X.sum(1)).flatten() > 0, :]
 
     return adata, adata_no_splicing
