@@ -1,11 +1,12 @@
-import numpy as np
-import scipy
+import re
+
 import matplotlib
 import matplotlib.pyplot as plt
 import networkx as nx
+import numpy as np
+import scipy
 from scipy.sparse import csr_matrix
 from scipy.sparse.csgraph import shortest_path
-import re
 
 from .DDRTree_py import DDRTree_py
 
@@ -50,7 +51,6 @@ def calculate_angle(o, y, x):
 
 
 def construct_velocity_tree_py(X1, X2):
-
     n = X1.shape[1]
 
     # merge two data with a given time
@@ -64,9 +64,7 @@ def construct_velocity_tree_py(X1, X2):
     gamma = 10
 
     # run DDRTree algorithm
-    W, Z, stree, Y, R, history = DDRTree_py(
-        X_all, maxIter=maxIter, eps=eps, sigma=sigma, gamma=gamma
-    )
+    W, Z, stree, Y, R, history = DDRTree_py(X_all, maxIter=maxIter, eps=eps, sigma=sigma, gamma=gamma)
 
     # draw velocity figure
 
@@ -113,7 +111,10 @@ def construct_velocity_tree_py(X1, X2):
             nodeid = u
             ve_nodeid = v
             shortest_mat = shortest_path(
-                csgraph=G, directed=False, indices=nodeid, return_predecessors=True
+                csgraph=G,
+                directed=False,
+                indices=nodeid,
+                return_predecessors=True,
             )
             velocity_path = []
             while ve_nodeid != nodeid:
@@ -196,9 +197,7 @@ def construct_velocity_tree_py(X1, X2):
         p1 = Y[:, row[tn]]
         p2 = Y[:, col[tn]]
         dp = p2 - p1
-        h = plt.quiver(
-            p1(1), p1(2), dp(1), dp(2), "LineWidth", 5
-        )  ###############need to plot it
+        h = plt.quiver(p1(1), p1(2), dp(1), dp(2), "LineWidth", 5)  ###############need to plot it
         set(h, "MaxHeadSize", 1e3, "AutoScaleFactor", 1)  #############
 
     for i in range(n):
