@@ -1,9 +1,10 @@
 import numpy as np
-from .DDRTree_py import DDRTree_py as DDRTree
+
+from .DDRTree_py import DDRTree
 from .utils import log1p_
 
 
-def Pseudotime(adata, layer="X", method="DDRTree", **kwargs):
+def Pseudotime(adata, layer="X", basis=None, method="DDRTree", **kwargs):
     """
 
     Parameters
@@ -18,8 +19,11 @@ def Pseudotime(adata, layer="X", method="DDRTree", **kwargs):
 
     """
 
-    X = adata.layers["X_" + layer].T if layer != "X" else adata.X.T
-    X = log1p_(adata, X)
+    if basis is None:
+        X = adata.layers["X_" + layer].T if layer != "X" else adata.X.T
+        X = log1p_(adata, X)
+    else:
+        X = adata.obsm["X_" + basis]
 
     DDRTree_kwargs = {
         "maxIter": 10,
@@ -61,15 +65,11 @@ def _cal_ncenter(ncells, ncells_limit=100):
     if ncells <= ncells_limit:
         return None
     else:
-        return np.round(
-            2 * ncells_limit * np.log(ncells) / (np.log(ncells) + np.log(ncells_limit))
-        )
+        return np.round(2 * ncells_limit * np.log(ncells) / (np.log(ncells) + np.log(ncells_limit)))
 
 
 # make this function to also calculate the directed graph between clusters:
-def compute_partition(
-    adata, transition_matrix, cell_membership, principal_g, group=None
-):
+def compute_partition(adata, transition_matrix, cell_membership, principal_g, group=None):
     """
 
     Parameters
