@@ -3,6 +3,7 @@ import numpy as np
 # ---------------------------------------------------------------------------------------------------
 # specificity related
 
+
 def specificity(percentage, perfect_specificity):
     """Calculate specificity"""
 
@@ -25,27 +26,32 @@ def shannon_entropy(p):
 
     if np.min(p) < 0 or np.sum(p) <= 0:
         return np.inf
-    p_norm = p[p > 0]/np.sum(p)
+    p_norm = p[p > 0] / np.sum(p)
 
-    return - np.sum(np.log(p_norm) * p_norm)
+    return -np.sum(np.log(p_norm) * p_norm)
 
 
 def JSdistVec(p, q):
     """Calculate the Jessen-Shannon distance for two probability distribution"""
 
     Jsdiv = shannon_entropy((p + q) / 2) - (shannon_entropy(p) + shannon_entropy(q)) / 2
-    if np.isinf(Jsdiv): Jsdiv = 1
-    if Jsdiv < 0: Jsdiv = 0
+    if np.isinf(Jsdiv):
+        Jsdiv = 1
+    if Jsdiv < 0:
+        Jsdiv = 0
     JSdist = np.sqrt(Jsdiv)
 
     return JSdist
 
+
 # ---------------------------------------------------------------------------------------------------
 # differential gene expression test related
 
+
 def fdr(p_vals):
     """calculate fdr_bh (Benjamini/Hochberg (non-negative))"""
     from scipy.stats import rankdata
+
     ranked_p_values = rankdata(p_vals)
     fdr = p_vals * len(p_vals) / ranked_p_values
     fdr[fdr > 1] = 1
