@@ -80,46 +80,23 @@ class moments:
         dx[self.ui] = la * ai - be * x[self.ui] - b * (x[self.ui] - x[self.ua])
         dx[self.wa] = (1 - la) * aa - be * x[self.wa] + a * (x[self.wi] - x[self.wa])
         dx[self.wi] = (1 - la) * ai - be * x[self.wi] - b * (x[self.wi] - x[self.wa])
-        dx[self.xa] = (
-            be * (1 - si) * x[self.ua] - ga * x[self.xa] + a * (x[self.xi] - x[self.xa])
-        )
-        dx[self.xi] = (
-            be * (1 - si) * x[self.ui] - ga * x[self.xi] - b * (x[self.xi] - x[self.xa])
-        )
-        dx[self.ya] = (
-            be * si * x[self.ua]
-            + be * x[self.wa]
-            - ga * x[self.ya]
-            + a * (x[self.yi] - x[self.ya])
-        )
-        dx[self.yi] = (
-            be * si * x[self.ui]
-            + be * x[self.wi]
-            - ga * x[self.yi]
-            - b * (x[self.yi] - x[self.ya])
-        )
+        dx[self.xa] = be * (1 - si) * x[self.ua] - ga * x[self.xa] + a * (x[self.xi] - x[self.xa])
+        dx[self.xi] = be * (1 - si) * x[self.ui] - ga * x[self.xi] - b * (x[self.xi] - x[self.xa])
+        dx[self.ya] = be * si * x[self.ua] + be * x[self.wa] - ga * x[self.ya] + a * (x[self.yi] - x[self.ya])
+        dx[self.yi] = be * si * x[self.ui] + be * x[self.wi] - ga * x[self.yi] - b * (x[self.yi] - x[self.ya])
 
         # second moments
-        dx[self.uu] = (
-            2 * la * self.fbar(aa * x[self.ua], ai * x[self.ui]) - 2 * be * x[self.uu]
-        )
-        dx[self.ww] = (
-            2 * (1 - la) * self.fbar(self.aa * x[self.wa], ai * x[self.wi])
-            - 2 * be * x[self.ww]
-        )
+        dx[self.uu] = 2 * la * self.fbar(aa * x[self.ua], ai * x[self.ui]) - 2 * be * x[self.uu]
+        dx[self.ww] = 2 * (1 - la) * self.fbar(self.aa * x[self.wa], ai * x[self.wi]) - 2 * be * x[self.ww]
         dx[self.xx] = 2 * be * (1 - si) * x[self.ux] - 2 * ga * x[self.xx]
-        dx[self.yy] = (
-            2 * si * be * x[self.uy] + 2 * be * x[self.wy] - 2 * ga * x[self.yy]
-        )
+        dx[self.yy] = 2 * si * be * x[self.uy] + 2 * be * x[self.wy] - 2 * ga * x[self.yy]
         dx[self.uw] = (
             la * self.fbar(aa * x[self.wa], ai * x[self.wi])
             + (1 - la) * self.fbar(aa * x[self.ua], ai * x[self.ui])
             - 2 * be * x[self.uw]
         )
         dx[self.ux] = (
-            la * self.fbar(aa * x[self.xa], ai * x[self.xi])
-            + be * (1 - si) * x[self.uu]
-            - (be + ga) * x[self.ux]
+            la * self.fbar(aa * x[self.xa], ai * x[self.xi]) + be * (1 - si) * x[self.uu] - (be + ga) * x[self.ux]
         )
         dx[self.uy] = (
             la * self.fbar(aa * x[self.ya], ai * x[self.yi])
@@ -229,19 +206,19 @@ class moments:
 
     def get_var_nu(self):
         c = self.get_nu()
-        return self.x[:, self.uu] + c - c ** 2
+        return self.x[:, self.uu] + c - c**2
 
     def get_var_nw(self):
         c = self.get_nw()
-        return self.x[:, self.ww] + c - c ** 2
+        return self.x[:, self.ww] + c - c**2
 
     def get_var_nx(self):
         c = self.get_nx()
-        return self.x[:, self.xx] + c - c ** 2
+        return self.x[:, self.xx] + c - c**2
 
     def get_var_ny(self):
         c = self.get_ny()
-        return self.x[:, self.yy] + c - c ** 2
+        return self.x[:, self.yy] + c - c**2
 
     def get_cov_ux(self):
         cu = self.get_nu()
@@ -531,17 +508,12 @@ class estimation:
         if method == "lhs":
             ret = self._lhsclassic(samples)
             for i in range(self.n_params):
-                ret[:, i] = (
-                    ret[:, i] * (self.ranges[i][1] - self.ranges[i][0])
-                    + self.ranges[i][0]
-                )
+                ret[:, i] = ret[:, i] * (self.ranges[i][1] - self.ranges[i][0]) + self.ranges[i][0]
         else:
             for n in range(samples):
                 for i in range(self.n_params):
                     r = random.rand()
-                    ret[n, i] = (
-                        r * (self.ranges[i][1] - self.ranges[i][0]) + self.ranges[i][0]
-                    )
+                    ret[n, i] = r * (self.ranges[i][1] - self.ranges[i][0]) + self.ranges[i][0]
         return ret
 
     def _lhsclassic(self, samples):
@@ -608,9 +580,7 @@ class estimation:
         x_data_norm = self.normalize_data(x_data)
         if bounds is None:
             bounds = (self.get_bound(0), self.get_bound(1))
-        popt, pcov = curve_fit(
-            self.f_curve_fit, t, x_data_norm.flatten(), p0=p0, bounds=bounds
-        )
+        popt, pcov = curve_fit(self.f_curve_fit, t, x_data_norm.flatten(), p0=p0, bounds=bounds)
         return popt, pcov
 
     def fit_lsq(
@@ -638,9 +608,7 @@ class estimation:
         X = []
         for i in range(n_p0):
             ret = least_squares(
-                lambda p: self.f_lsq(
-                    p, t, x_data_norm.flatten(), method, experiment_type
-                ),
+                lambda p: self.f_lsq(p, t, x_data_norm.flatten(), method, experiment_type),
                 p0[i],
                 bounds=bounds,
             )
