@@ -4,9 +4,7 @@ import numpy as np
 from scipy.interpolate import griddata
 
 
-def path_integral(
-    VecFnc, x_lim, y_lim, xyGridSpacing, dt=1e-2, tol=1e-2, numTimeSteps=1400
-):
+def path_integral(VecFnc, x_lim, y_lim, xyGridSpacing, dt=1e-2, tol=1e-2, numTimeSteps=1400):
     """A deterministic map of Waddington’s epigenetic landscape for cell fate specification
     Sudin Bhattacharya, Qiang Zhang and Melvin E. Andersen
 
@@ -70,9 +68,7 @@ def path_integral(
     y_path = np.zeros((numPaths, numTimeSteps))  # y-coord. along path
     pot_path = np.zeros((numPaths, numTimeSteps))  # pot. along path
 
-    path_tag = np.ones(
-        (numPaths, 1), dtype="int"
-    )  # tag for given path (to denote basin of attraction)
+    path_tag = np.ones((numPaths, 1), dtype="int")  # tag for given path (to denote basin of attraction)
     # ** initialized to 1 for all paths **
 
     # Initialize "Path counter" to 1
@@ -140,9 +136,7 @@ def path_integral(
                 y_path[path_counter, n_steps] = y_p
 
                 # update "potential"
-                dPot = (
-                    -(dxdt) * dx - (dydt) * dy
-                )  # signs ensure that "potential" decreases as "velocity" increases
+                dPot = -(dxdt) * dx - (dydt) * dy  # signs ensure that "potential" decreases as "velocity" increases
                 Pot = Pot_old + dPot
                 pot_path[path_counter, n_steps] = Pot
 
@@ -159,24 +153,18 @@ def path_integral(
             if path_counter == 0:
                 # record attractor of first path and its coords
                 num_attractors = num_attractors + 1
-                current_att_num_X_Y = np.array(
-                    [num_attractors, x_p, y_p]
-                )  # create array
+                current_att_num_X_Y = np.array([num_attractors, x_p, y_p])  # create array
                 attractors_num_X_Y = (
                     np.vstack((attractors_num_X_Y, current_att_num_X_Y))
                     if attractors_num_X_Y is not None
                     else np.array([current_att_num_X_Y])
                 )  # append array (vertically)
                 attractors_pot = (
-                    np.vstack((attractors_pot, Pot))
-                    if attractors_pot is not None
-                    else np.array([Pot])
+                    np.vstack((attractors_pot, Pot)) if attractors_pot is not None else np.array([Pot])
                 )  # append attractor potentials to array (vertically)
                 path_tag[path_counter] = num_attractors - 1  # initialize path tag
                 numPaths_att = (
-                    np.vstack((numPaths_att, 1))
-                    if numPaths_att is not None
-                    else np.array([1])
+                    np.vstack((numPaths_att, 1)) if numPaths_att is not None else np.array([1])
                 )  # append to array (vertically)
 
             else:
@@ -201,7 +189,7 @@ def path_integral(
 
                 # check if the current path *ended* in a different point compared to previous path (x-y grid spacing used
                 # as a "tolerance" for distance)
-                if endPt_dist_sqr > (2 * (xyGridSpacing ** 2)):
+                if endPt_dist_sqr > (2 * (xyGridSpacing**2)):
 
                     # --- check if this "different" attractor has been identified before
                     new_attr_found = 1
@@ -209,9 +197,7 @@ def path_integral(
                     for k in range(num_attractors):
                         x_att = attractors_num_X_Y[k, 1]
                         y_att = attractors_num_X_Y[k, 2]
-                        if (abs(x_p - x_att) < xyGridSpacing) and (
-                            abs(y_p - y_att) < xyGridSpacing
-                        ):
+                        if (abs(x_p - x_att) < xyGridSpacing) and (abs(y_p - y_att) < xyGridSpacing):
                             # this attractor has been identified before
                             new_attr_found = 0
                             path_tag[path_counter] = k  # DOUBLE CHECK ***
@@ -220,20 +206,22 @@ def path_integral(
 
                     if new_attr_found == 1:
                         num_attractors = num_attractors + 1
-                        current_att_num_X_Y = [num_attractors, x_p, y_p]  # create array
+                        current_att_num_X_Y = [
+                            num_attractors,
+                            x_p,
+                            y_p,
+                        ]  # create array
                         attractors_num_X_Y = np.vstack(
                             (attractors_num_X_Y, current_att_num_X_Y)
                         )  # append array (vertically)
                         path_tag[path_counter] = num_attractors - 1  # DOUBLE CHECK **
-                        numPaths_att = np.vstack(
-                            (numPaths_att, 1)
-                        )  # append to array (vertically)
+                        numPaths_att = np.vstack((numPaths_att, 1))  # append to array (vertically)
                         attractors_pot = np.vstack(
                             (attractors_pot, Pot)
                         )  # append attractor potentials to array (vertically)
 
                         # check if start points of current and previous paths are "adjacent" - if so, assign separatrix
-                        if startPt_dist_sqr < (2 * (xyGridSpacing ** 2)):
+                        if startPt_dist_sqr < (2 * (xyGridSpacing**2)):
                             curr_sepx = [
                                 path_tag[path_counter - 1],
                                 path_tag[path_counter],
@@ -258,16 +246,14 @@ def path_integral(
                             attr1 = sepx_old_new_pathNum[k, 0]
                             attr2 = sepx_old_new_pathNum[k, 1]
 
-                            if (path_tag[path_counter - 1] == attr1) or (
-                                path_tag[path_counter - 1] == attr2
-                            ):
+                            if (path_tag[path_counter - 1] == attr1) or (path_tag[path_counter - 1] == attr2):
                                 # this attractor has been identified before
                                 prev_attr_new = 0
                                 break  # exit for-loop
 
                         if prev_attr_new == 1:
                             # check if start points of current and previous paths are "adjacent"  - if so, assign separatrix
-                            if startPt_dist_sqr < (2 * (xyGridSpacing ** 2)):
+                            if startPt_dist_sqr < (2 * (xyGridSpacing**2)):
                                 curr_sepx = [
                                     path_tag[path_counter - 1],
                                     path_tag[path_counter],
@@ -321,7 +307,7 @@ def alignment(
     grid=100,
     interpolation_method="linear",
 ):
-    """ Align potential values so all path-potentials end up at same global min and then generate potential surface with
+    """Align potential values so all path-potentials end up at same global min and then generate potential surface with
     interpolation on a grid.
 
     Parameters
