@@ -1,24 +1,35 @@
+from warnings import warn
+
 import numpy as np
 import scipy as sp
 import scipy.optimize
 
-from .Bhattacharya import path_integral, alignment
+from ..tools.sampling import lhsclassic
 from .Ao import Ao_pot_map
+from .Bhattacharya import alignment, path_integral
+from .topography import FixedPoints
+from .utils import is_outside_domain
+from .Wang import Wang_action, Wang_LAP
 
 # import autograd.numpy as autonp
 # from autograd import grad, jacobian # calculate gradient and jacobian
 
-from .Wang import Wang_action, Wang_LAP
+
 # the LAP method should be rewritten in TensorFlow/PyTorch using optimization with SGD
 
-from .topography import FixedPoints
-from .utils import is_outside_domain
-from ..tools.sampling import lhsclassic
-from warnings import warn
 
-def search_fixed_points(func, domain, x0, x0_method='lhs', 
-    reverse=False, return_x0=False, fval_tol=1e-8, 
-    remove_outliers=True, ignore_fsolve_err=False, **fsolve_kwargs):
+def search_fixed_points(
+    func,
+    domain,
+    x0,
+    x0_method="lhs",
+    reverse=False,
+    return_x0=False,
+    fval_tol=1e-8,
+    remove_outliers=True,
+    ignore_fsolve_err=False,
+    **fsolve_kwargs
+):
     import numdifftools as nda
 
     func_ = (lambda x: -func(x)) if reverse else func
@@ -27,18 +38,18 @@ def search_fixed_points(func, domain, x0, x0_method='lhs',
     if np.isscalar(x0):
         n = x0
 
-        if k > 2 and x0_method == 'grid':
-            warn('The dimensionality is too high (%dD). Using lhs instead...'%k)
-            x0_method = 'lhs'
-        
-        if x0_method == 'lhs':
-            print('Sampling initial points using latin hypercube sampling...')
-            x0 = lhsclassic(n, k) 
-        elif x0_method == 'grid':
-            print('Sampling initial points on a grid...')
+        if k > 2 and x0_method == "grid":
+            warn("The dimensionality is too high (%dD). Using lhs instead..." % k)
+            x0_method = "lhs"
+
+        if x0_method == "lhs":
+            print("Sampling initial points using latin hypercube sampling...")
+            x0 = lhsclassic(n, k)
+        elif x0_method == "grid":
+            print("Sampling initial points on a grid...")
             pass
         else:
-            print('Sampling initial points randomly (uniform distribution)...')
+            print("Sampling initial points randomly (uniform distribution)...")
             x0 = np.random.rand(n, k)
 
         x0 = x0 * (domain[1] - domain[0]) + domain[0]
@@ -46,39 +57,45 @@ def search_fixed_points(func, domain, x0, x0_method='lhs',
     fp = FixedPoints()
     succeed = 0
     for i in range(len(x0)):
-        x, fval_dict, ier, mesg = sp.optimize.fsolve(
-                func_, x0[i], full_output=True, **fsolve_kwargs)
+        x, fval_dict, ier, mesg = sp.optimize.fsolve(func_, x0[i], full_output=True, **fsolve_kwargs)
 
         if ignore_fsolve_err:
             ier = 1
         if fval_dict["fvec"].dot(fval_dict["fvec"]) > fval_tol and ier == 1:
             ier = -1
-            mesg = 'Function evaluated at the output is larger than the tolerance.'
+            mesg = "Function evaluated at the output is larger than the tolerance."
         elif remove_outliers and is_outside_domain(x, domain) and ier == 1:
             ier = -2
-            mesg = 'The output is outside the domain.'
+            mesg = "The output is outside the domain."
 
         if ier == 1:
             jacobian_mat = nda.Jacobian(func_)(np.array(x))
             fp.add_fixed_points([x], [jacobian_mat])
             succeed += 1
         else:
-            #jacobian_mat = nda.Jacobian(func_)(np.array(x))
-            #fp.add_fixed_points([x], [jacobian_mat])
-            print('Solution not found: ' + mesg)
+            # jacobian_mat = nda.Jacobian(func_)(np.array(x))
+            # fp.add_fixed_points([x], [jacobian_mat])
+            print("Solution not found: " + mesg)
+
+    print("%d/%d solutions found." % (succeed, len(x0)))
 
-    print('%d/%d solutions found.'%(succeed, len(x0)))
-        
     if return_x0:
         return fp, x0
     else:
         return fp
-    
+
 
 def gen_fixed_points(
-    func, auto_func, dim_range, RandNum, EqNum, reverse=False, grid_num=50, x_ini=None
+    func,
+    auto_func,
+    dim_range,
+    RandNum,
+    EqNum,
+    reverse=False,
+    grid_num=50,
+    x_ini=None,
 ):
-    """ Calculate the fixed points of (learned) vector field function . Classify the fixed points into classes of stable and saddle points
+    """Calculate the fixed points of (learned) vector field function . Classify the fixed points into classes of stable and saddle points
     based on the eigenvalue of the Jacobian on the point.
 
     Arguments
@@ -121,14 +138,10 @@ def gen_fixed_points(
 
     if x_ini is None and EqNum < 4:
         _min, _max = [dim_range[0]] * EqNum, [dim_range[1]] * EqNum
-        Grid_list = np.meshgrid(
-            *[np.linspace(i, j, grid_num) for i, j in zip(_min, _max)]
-        )
+        Grid_list = np.meshgrid(*[np.linspace(i, j, grid_num) for i, j in zip(_min, _max)])
         x_ini = np.array([i.flatten() for i in Grid_list]).T
 
-    RandNum = (
-        RandNum if x_ini is None else x_ini.shape[0]
-    )  # RandNum set to the manually input steady state estimates
+    RandNum = RandNum if x_ini is None else x_ini.shape[0]  # RandNum set to the manually input steady state estimates
     FixedPoint = np.zeros((RandNum, EqNum))
     Type = np.zeros((RandNum, 1))
 
@@ -141,13 +154,8 @@ def gen_fixed_points(
 
     if x_ini is None:
         for time in range(RandNum):
-            x0 = (
-                np.random.uniform(0, 1, EqNum) * (dim_range[1] - dim_range[0])
-                + dim_range[0]
-            )
-            x, fval_dict, _, _ = sp.optimize.fsolve(
-                func_, x0, maxfev=450000, xtol=FixedPointConst, full_output=True
-            )
+            x0 = np.random.uniform(0, 1, EqNum) * (dim_range[1] - dim_range[0]) + dim_range[0]
+            x, fval_dict, _, _ = sp.optimize.fsolve(func_, x0, maxfev=450000, xtol=FixedPointConst, full_output=True)
             # fjac: the orthogonal matrix, q, produced by the QR factorization of the final approximate Jacobian matrix,
             # stored column wise; r: upper triangular matrix produced by QR factorization of the same matrix
             # if auto_func is None:
@@ -172,9 +180,7 @@ def gen_fixed_points(
     else:
         for time in range(x_ini.shape[0]):
             x0 = x_ini[time, :]
-            x, fval_dict, _, _ = sp.optimize.fsolve(
-                func_, x0, maxfev=450000, xtol=FixedPointConst, full_output=True
-            )
+            x, fval_dict, _, _ = sp.optimize.fsolve(func_, x0, maxfev=450000, xtol=FixedPointConst, full_output=True)
             # fjac: the orthogonal matrix, q, produced by the QR factorization of the final approximate Jacobian matrix,
             # stored column wise; r: upper triangular matrix produced by QR factorization of the same matrix
             # if auto_func is None:
@@ -203,9 +209,7 @@ def gen_fixed_points(
         elif Type[time] == 1:
             for i in range(StableNum + 1):
                 temp = StablePoint[i, :] - FixedPoint[time, :]
-                if (
-                    i is not StableNum + 1 and temp.dot(temp) < ZeroConst
-                ):  # avoid duplicated attractors
+                if i is not StableNum + 1 and temp.dot(temp) < ZeroConst:  # avoid duplicated attractors
                     StableTimes[i] = StableTimes[i] + 1
                     break
             if i is StableNum:
@@ -215,9 +219,7 @@ def gen_fixed_points(
         elif Type[time] == -1:
             for i in range(SaddleNum + 1):
                 temp = SaddlePoint[i, :] - FixedPoint[time, :]
-                if (
-                    i is not SaddlePoint and temp.dot(temp) < ZeroConst
-                ):  # avoid duplicated saddle point
+                if i is not SaddlePoint and temp.dot(temp) < ZeroConst:  # avoid duplicated saddle point
                     SaddleTimes[i] = SaddleTimes[i] + 1
                     break
             if i is SaddleNum:
@@ -252,8 +254,8 @@ def gen_gradient(dim, N, Function, DiffusionMatrix):
         A matrix consists of the coordinates of the unstable steady state
     """
 
-    from sympy import MatrixSymbol, Identity, symbols, Matrix, simplify
     from StringFunction import StringFunction
+    from sympy import Identity, Matrix, MatrixSymbol, simplify, symbols
 
     N = N + 1
     X = MatrixSymbol("x", dim, N)
@@ -269,12 +271,7 @@ def gen_gradient(dim, N, Function, DiffusionMatrix):
     for k in np.arange(1, N):  # equation 18
         k
         # S+1/4*dt*((X(:,k)-X(:,k-1))/dt-ODE(X(:,k-1))).'*(DiffusionMatrix(X(:,k-1)))^(-1)*((X(:,k)-X(:,k-1))/dt-ODE(X(:,k-1)));
-        t1 = (
-            1
-            / 4
-            * dt
-            * ((X[:, k] - X[:, k - 1]) / dt - Matrix(Function(X[:, k - 1]))).T
-        )
+        t1 = 1 / 4 * dt * ((X[:, k] - X[:, k - 1]) / dt - Matrix(Function(X[:, k - 1]))).T
 
         t2 = Matrix(np.linalg.inv(DiffusionMatrix(X[:, k - 1])))
         t3 = (X[:, k] - X[:, k - 1]) / dt - Matrix(Function(X[:, k - 1]))
@@ -315,13 +312,13 @@ def IntGrad(points, Function, DiffusionMatrix, dt):
 
     Arguments
     ---------
-        points: 'np.ndarray'
+        points: :class:`~numpy.ndarray`
             The sampled points in the state space used to calculate the action.
-        Function: 'function'
+        Function: function
             The (learned) vector field function.
-        DiffusionMatrix: 'function'
+        DiffusionMatrix: function
             The function that returns diffusion matrix which can be dependent on the variables (for example, genes)
-        dt: 'function'
+        dt: 'float'
             The time interval used in calculating action
 
     Returns
@@ -332,15 +329,9 @@ def IntGrad(points, Function, DiffusionMatrix, dt):
 
     integral = 0
     for k in np.arange(1, points.shape[1]):
-        Tmp = (points[:, k] - points[:, k - 1]).reshape((-1, 1)) / dt - Function(
-            points[:, k - 1]
-        ).reshape((-1, 1))
+        Tmp = (points[:, k] - points[:, k - 1]).reshape((-1, 1)) / dt - Function(points[:, k - 1]).reshape((-1, 1))
         integral = (
-            integral
-            + (Tmp.T)
-            .dot(np.linalg.matrix_power(DiffusionMatrix(points[:, k - 1]), -1))
-            .dot(Tmp)
-            * dt
+            integral + (Tmp.T).dot(np.linalg.matrix_power(DiffusionMatrix(points[:, k - 1]), -1)).dot(Tmp) * dt
         )  # part of the Eq. 18 in Sci. Rep. paper
     integral = integral / 4
 
@@ -348,12 +339,12 @@ def IntGrad(points, Function, DiffusionMatrix, dt):
 
 
 def DiffusionMatrix(x):
-    """ Diffusion matrix can be variable dependent
+    """Diffusion matrix can be variable dependent
 
     Arguments
     ---------
-        x: 'np.ndarray'
-            The matrix of sampled points (cells) in the (gene expression) state space.
+        x: :class:`~numpy.ndarray`
+            The matrix of sampled points (cells) in the (gene expression) state space. A
 
     Returns
     -------
@@ -366,10 +357,9 @@ def DiffusionMatrix(x):
     return out
 
 
-# rewrite action in TF with SGD
 def action(n_points, tmax, point_start, point_end, boundary, Function, DiffusionMatrix):
-    """It calculates the minimized action value given an intial path, ODE, and diffusion matrix. The minimization is
-    realized by scipy.optimize.Bounds function in python (withnot using the gradient of the action function).
+    """It calculates the minimized action value given an initial path, ODE, and diffusion matrix. The minimization is
+    realized by scipy.optimize.Bounds function in python (without using the gradient of the action function).
 
     Arguments
     ---------
@@ -377,27 +367,29 @@ def action(n_points, tmax, point_start, point_end, boundary, Function, Diffusion
             The number of points along the least action path.
         tmax: 'int'
             The value at maximum t.
-        point_start: 'np.ndarray'
+        point_start: :class:`~numpy.ndarray`
             The matrix for storing the coordinates (gene expression configuration) of the start point (initial cell state).
-        point_end: 'np.ndarray'
+        point_end: :class:`~numpy.ndarray`
             The matrix for storing the coordinates (gene expression configuration) of the end point (terminal cell state).
-        Function: 'function'
+        boundary: :class:`~numpy.ndarray`
+            Not used.
+        Function: function
             The (reconstructed) vector field function.
-        DiffusionMatrix: 'function'
+        DiffusionMatrix: function
             The function that returns the diffusion matrix which can variable (for example, gene) dependent.
 
     Returns
     -------
-    fval: 'np.ndarray'
+    fval: :class:`~numpy.ndarray`
         The action value for the learned least action path.
-    output_path: 'np.ndarray'
+    output_path: :class:`~numpy.ndarray`
         The least action path learned
     """
 
     dim = point_end.shape[0]  # genes x cells
     dt = tmax / n_points
     lambda_f = lambda x: IntGrad(
-        np.hstack((point_start, x.reshape((2, -1)), point_end)),
+        np.hstack((point_start, x.reshape((dim, -1)), point_end)),
         Function,
         DiffusionMatrix,
         dt,
@@ -406,10 +398,7 @@ def action(n_points, tmax, point_start, point_end, boundary, Function, Diffusion
     # initial path as a line connecting start point and end point point_start*ones(1,n_points+1)+(point_end-point_start)*(0:tmax/n_points:tmax)/tmax;
     initpath = (
         point_start.dot(np.ones((1, n_points + 1)))
-        + (point_end - point_start).dot(
-            np.linspace(0, tmax, n_points + 1, endpoint=True).reshape(1, -1)
-        )
-        / tmax
+        + (point_end - point_start).dot(np.linspace(0, tmax, n_points + 1, endpoint=True).reshape(1, -1)) / tmax
     )
 
     Bounds = scipy.optimize.Bounds(
@@ -460,7 +449,7 @@ def Potential(adata, DiffMat=None, method="Ao", **kwargs):
 
     """
 
-    Function = adata.uns["VecFld"]["VecFld"]
+    Function = adata.uns["VecFld"]
     DiffMat = DiffusionMatrix if DiffMat is None else DiffMat
     pot = Pot(Function, DiffMat, **kwargs)
     pot.fit(method=method)
@@ -481,7 +470,7 @@ class Pot:
         stable=None,
         saddle=None,
     ):
-        """ It implements the least action method to calculate the potential values of fixed points for a given SDE (stochastic
+        """It implements the least action method to calculate the potential values of fixed points for a given SDE (stochastic
         differential equation) model. The function requires the vector field function and a diffusion matrix. This code is based
         on the MATLAB code from Ruoshi Yuan and Ying Tang. Potential landscape of high dimensional nonlinear stochastic dynamics with
         large noise. Y Tang, R Yuan, G Wang, X Zhu, P Ao - Scientific reports, 2017
@@ -584,9 +573,7 @@ class Pot:
 
         if method == "Ao":
             X = adata.obsm["X_" + basis]
-            X, U, P, vecMat, S, A = Ao_pot_map(
-                self.VecFld["Function"], X, D=self.VecFld["DiffusionMatrix"]
-            )
+            X, U, P, vecMat, S, A = Ao_pot_map(self.VecFld["Function"], X, D=self.VecFld["DiffusionMatrix"])
 
             adata.uns["grid_Pot_" + basis] = {
                 "Xgrid": X,
@@ -596,15 +583,7 @@ class Pot:
                 "A": A,
             }
         elif method == "Bhattacharya":
-            (
-                numPaths,
-                numTimeSteps,
-                pot_path,
-                path_tag,
-                attractors_pot,
-                x_path,
-                y_path,
-            ) = path_integral(
+            (numPaths, numTimeSteps, pot_path, path_tag, attractors_pot, x_path, y_path,) = path_integral(
                 self.VecFld["Function"],
                 x_lim=x_lim,
                 y_lim=y_lim,
@@ -637,15 +616,7 @@ class Pot:
                 self.VecFld["Function"],
                 self.VecFld["DiffusionMatrix"],
             )
-            (
-                boundary,
-                n_points,
-                fixed_point_only,
-                find_fixed_points,
-                refpoint,
-                stable,
-                saddle,
-            ) = (
+            (boundary, n_points, fixed_point_only, find_fixed_points, refpoint, stable, saddle,) = (
                 self.parameters["boundary"],
                 self.parameters["n_points"],
                 self.parameters["fixed_point_only"],
@@ -660,7 +631,11 @@ class Pot:
             if (stable is None and saddle is None) or find_fixed_points is True:
                 print("stable is ", stable)
                 stable, saddle = gen_fixed_points(
-                    Function, auto_func=False, dim_range=range, RandNum=100, EqNum=2
+                    Function,
+                    auto_func=False,
+                    dim_range=range,
+                    RandNum=100,
+                    EqNum=2,
                 )  # gen_fixed_points(vector_field_function, auto_func = None, dim_range = [-25, 25], RandNum = 5000, EqNum = 2, x_ini = None)
 
             print("stable 2 is ", stable)
@@ -687,11 +662,7 @@ class Pot:
                 points = np.vstack((X.flatten(), Y.flatten()))
                 I = range(points.shape[1])
 
-            for (
-                ind
-            ) in (
-                I
-            ):  # action(n_points,tmax,point_start,point_end, boundary, Function, DiffusionMatrix):
+            for ind in I:  # action(n_points,tmax,point_start,point_end, boundary, Function, DiffusionMatrix):
                 print("current ind is ", ind)
                 lav, lap = action(
                     TotalPoints,
